MEMORY
{
 ITCRAM (rwx) :  ORIGIN = 0x40090000, LENGTH = 64K
 DTCRAM (xrw) :  ORIGIN = 0x3FFF0000, LENGTH = 64K
}

REGION_ALIAS("RAM",  DTCRAM);
REGION_ALIAS("RAM2", DTCRAM);

REGION_ALIAS("TEXT", ITCRAM);

__stack = ORIGIN(RAM) + LENGTH(RAM);
__Main_Stack_Size = 1024 ;

PROVIDE ( _Main_Stack_Size = __Main_Stack_Size ) ;
__Main_Stack_Limit = __stack  - __Main_Stack_Size ;
PROVIDE ( _Main_Stack_Limit = __Main_Stack_Limit ) ;

_Minimum_Stack_Size = 256 ;

PROVIDE ( _Heap_Begin = _end_noinit ) ;
PROVIDE ( _Heap_Limit = __stack - __Main_Stack_Size ) ;

ENTRY(_start)

SECTIONS
{
  .resetentry : ALIGN(4)
  {
    KEEP(*(.resetentry))
    KEEP(*(.startup))

    *(.*Vector.literal)

    *(.UserEnter.literal);
    *(.UserEnter.text);
    . = ALIGN (16);
    *(.entry.text)
    *(.init.literal)
    *(.init)

  } > TEXT

  .text : ALIGN(4)
  {
    *(.literal .literal.* )

	KEEP(*(.init))
	KEEP(*(.fini))

	. = ALIGN(4);

	PROVIDE_HIDDEN (__preinit_array_start = .);
	KEEP(*(.preinit_array_sysinit .preinit_array_sysinit.*))
	KEEP(*(.preinit_array_platform .preinit_array_platform.*))
	KEEP(*(.preinit_array .preinit_array.*))
	PROVIDE_HIDDEN (__preinit_array_end = .);

	. = ALIGN(4);

	PROVIDE_HIDDEN (__init_array_start = .);
	KEEP(*(SORT(.init_array.*)))
	KEEP(*(.init_array))
	PROVIDE_HIDDEN (__init_array_end = .);

	. = ALIGN(4);

	PROVIDE_HIDDEN (__fini_array_start = .);
	KEEP(*(SORT(.fini_array.*)))
	KEEP(*(.fini_array))
	PROVIDE_HIDDEN (__fini_array_end = .);

	*(.rela.iplt)
	*(.rela.plt)

	*(.text .text.* )

	*(.text.unlikely .text.*_unlikely .text.unlikely.*)
	*(.text.exit .text.exit.*)
	*(.text.startup .text.startup.*)
	*(.text.hot .text.hot.*)
	*(SORT(.text.sorted.*))
	*(.stub)
	*(.gnu.warning)

	*(.srodata .srodata.*)
	*(.rodata .rodata.* .constdata .constdata.* .gnu.linkonce.r.*)
	*(.ARM.extab* .gnu.linkonce.armextab.*)
	*(vtable)

	KEEP(*(.eh_frame*))
	*(.gnu.linkonce.t.*)

	*(.glue_7) *(.glue_7t)
	*(.gcc_except_table)

	. = ALIGN(4);
	KEEP (*crtbegin.o(.ctors))
	KEEP (*(EXCLUDE_FILE (*crtend.o) .ctors))
	KEEP (*(SORT(.ctors.*)))
	KEEP (*crtend.o(.ctors))

  } > TEXT

  . = ALIGN(4);
  _etext = .;
  __etext = .;

  .data_DTCRAM  : ALIGN(4)  { *(.data_DTCRAM) } > DTCRAM  AT > TEXT
  .data_ITCRAM  : ALIGN(4)  { *(.data_ITCRAM) } > ITCRAM  AT > TEXT
  .data_RAM2    : ALIGN(4)  { *(.data_RAM2) }   > RAM2    AT > TEXT
  .data : ALIGN(4)
  {
	*(.data_begin .data_begin.*)
	*(.data .data.* .gnu.linkonce.d.*)
	*(.data_end .data_end.*)

	*(.sdata2 .sdata2.* .gnu.linkonce.s2.*)

	SORT(CONSTRUCTORS)

	*(.srodata.cst16) *(.srodata.cst8) *(.srodata.cst4) *(.srodata.cst2) *(.srodata .srodata.*)
	*(.sdata .sdata.* .gnu.linkonce.s.*)
	*(.tdata .tdata.* .gnu.linkonce.td.*)
	
    . = ALIGN(4);
  } > RAM AT > TEXT

  __app_image_end = LOADADDR(.data) + SIZEOF(.data);

  __global_pointer$ = ADDR(.data) + 0x800;

  .bss_DTCRAM (NOLOAD) : ALIGN(4) { *(.bss_DTCRAM) } > DTCRAM
  .bss_ITCRAM (NOLOAD) : ALIGN(4) { *(.bss_ITCRAM) } > ITCRAM
  .bss_RAM2   (NOLOAD) : ALIGN(4) { *(.bss_RAM2) }   > RAM2
  .bss (NOLOAD) : ALIGN(4)
  {
	*(.bss_begin .bss_begin.*)
	*(.bss .bss.*)
	*(.gnu.linkonce.b.*)
	*(COMMON)
	*(.bss_end .bss_end.*)

	*(.dynsbss)
	*(.sbss .sbss.* .gnu.linkonce.sb.*)
	*(.scommon)
	*(.dynbss)

  } > RAM

  .noinit_DTCRAM (NOLOAD) : ALIGN(4)  { *(.noinit_DTCRAM) } > DTCRAM
  .noinit_ITCRAM (NOLOAD) : ALIGN(4)  { *(.noinit_ITCRAM) } > ITCRAM
  .noinit_RAM2   (NOLOAD) : ALIGN(4)  { *(.noinit_RAM2) }   > RAM2
  .noinit (NOLOAD) : ALIGN(4)
  {
    _noinit = .;
    *(.noinit .noinit.*)
    . = ALIGN(4) ;
    _end_noinit = .;
  } > RAM

	PROVIDE ( end = _end_noinit ); /* was _ebss */
	PROVIDE ( _end = _end_noinit );
	PROVIDE ( __end = _end_noinit );
	PROVIDE ( __end__ = _end_noinit );

	._check_stack : ALIGN(4)
	{
		. = . + _Minimum_Stack_Size ;
	} > RAM

	DISCARD :
	{
	 libc.a ( * )
	 libm.a ( * )
	 libgcc.a ( * )
	}

	/* Stabs debugging sections.  */
	.stab          0 : { *(.stab) }
	.stabstr       0 : { *(.stabstr) }
	.stab.excl     0 : { *(.stab.excl) }
	.stab.exclstr  0 : { *(.stab.exclstr) }
	.stab.index    0 : { *(.stab.index) }
	.stab.indexstr 0 : { *(.stab.indexstr) }
	.comment       0 : { *(.comment) }
	/*
	 * DWARF debug sections.
	 * Symbols in the DWARF debugging sections are relative to the beginning
	 * of the section so we begin them at 0.
	 */
	/* DWARF 1 */
	.debug          0 : { *(.debug) }
	.line           0 : { *(.line) }
	/* GNU DWARF 1 extensions */
	.debug_srcinfo  0 : { *(.debug_srcinfo) }
	.debug_sfnames  0 : { *(.debug_sfnames) }
	/* DWARF 1.1 and DWARF 2 */
	.debug_aranges  0 : { *(.debug_aranges) }
	.debug_pubnames 0 : { *(.debug_pubnames) }
	/* DWARF 2 */
	.debug_info     0 : { *(.debug_info .gnu.linkonce.wi.*) }
	.debug_abbrev   0 : { *(.debug_abbrev) }
	.debug_line     0 : { *(.debug_line) }
	.debug_frame    0 : { *(.debug_frame) }
	.debug_str      0 : { *(.debug_str) }
	.debug_loc      0 : { *(.debug_loc) }
	.debug_macinfo  0 : { *(.debug_macinfo) }
	/* SGI/MIPS DWARF 2 extensions */
	.debug_weaknames 0 : { *(.debug_weaknames) }
	.debug_funcnames 0 : { *(.debug_funcnames) }
	.debug_typenames 0 : { *(.debug_typenames) }
	.debug_varnames  0 : { *(.debug_varnames) }
	/* DWARF 3 */
	.debug_pubtypes 0 : { *(.debug_pubtypes) }
	.debug_ranges   0 : { *(.debug_ranges) }
	/* DWARF Extension.  */
	.debug_macro    0 : { *(.debug_macro) }
	.debug_addr     0 : { *(.debug_addr) }
	.gnu.attributes 0 : { KEEP (*(.gnu.attributes)) }

	/DISCARD/ : { *(.note.GNU-stack) *(.gnu_debuglink) *(.gnu.lto_*) }	
}
